@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -21,7 +21,7 @@ include("WidgetFactory.js");
 include("scripts/Widgets/OptionsToolBar/OptionsToolBar.js");
 
 if (RSettings.isGuiEnabled()) {
-    if (new QFileInfo(autoPath("scripts/Widgets/CadToolBar/CadToolBar.js")).exists()) {
+    if (exists("scripts/Widgets/CadToolBar/CadToolBar.js")) {
         include("scripts/Widgets/CadToolBar/ColumnLayout.js");
         include("scripts/Widgets/CadToolBar/CadToolBar.js");
     }
@@ -58,6 +58,7 @@ function EAction(guiAction) {
 }
 
 EAction.prototype = new RActionAdapter();
+EAction.includeBasePath = includeBasePath;
 
 // some commonly used translated strings:
 EAction.trBack = qsTr("Back");
@@ -157,6 +158,8 @@ EAction.prototype.finishEvent = function() {
     if (!isNull(di)) {
         di.repaintViews();
     }
+
+    this.setCommandPrompt();
 };
 
 /**
@@ -624,6 +627,10 @@ EAction.prototype.showDialog = function() {
 
     // sync invisible part of options tool bar with dialog:
     for (i=0; i<noSyncWidgets.length; i++) {
+        if (isDeleted(noSyncWidgets[i])) {
+            continue;
+        }
+
         noSyncWidgets[i].setProperty("Loaded", true);
     }
 
@@ -631,6 +638,10 @@ EAction.prototype.showDialog = function() {
     WidgetFactory.restoreState(optionsToolBar, this.settingsGroup, this);
 
     for (i=0; i<noSyncWidgets.length; i++) {
+        if (isDeleted(noSyncWidgets[i])) {
+            continue;
+        }
+
         noSyncWidgets[i].setProperty("Loaded", false);
     }
 
@@ -692,14 +703,19 @@ EAction.prototype.keyPressEvent = function(event) {
         return;
     }
 
-    if ((event.key() >= Qt.Key_0.valueOf() && event.key() <= Qt.Key_9.valueOf()) || event.key() === Qt.Key_Period.valueOf()) {
-        // number or dot entered:
-        // give focus to first input widget in options tool bar and set text to number entered:
-        var w = OptionsToolBar.getFirstInputWidget();
-        if (!isNull(w)) {
-            w.setFocus(Qt.OtherFocusReason);
-            w.text = String.fromCharCode(event.key());
-            return;
+    // prevent focus shift when entering keycode (e.g. "D2"):
+    var appWin = EAction.getMainWindow();
+    if (appWin.getKeyLog().length===0) {
+        if ((event.key() >= Qt.Key_0.valueOf() && event.key() <= Qt.Key_9.valueOf()) || event.key() === Qt.Key_Period.valueOf()) {
+            // number or dot entered:
+            // give focus to first input widget in options tool bar and set text to number entered:
+            var w = OptionsToolBar.getFirstInputWidget();
+            if (!isNull(w)) {
+                w.setFocus(Qt.OtherFocusReason);
+                w.text = String.fromCharCode(event.key());
+                event.accept();
+                return;
+            }
         }
     }
 
@@ -721,15 +737,15 @@ EAction.prototype.setCrosshairCursor = function() {
     if (isNull(EAction.crossCursor)) {
         var bitmap, mask
         if (RSettings.getDevicePixelRatio()===2 && RS.getSystemId()!=="osx") {
-            bitmap = new QBitmap("scripts/CrosshairCursor@2x.png", "PNG");
-            mask = new QBitmap("scripts/CrosshairCursorMask@2x.png", "PNG");
+            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor@2x.png", "PNG");
+            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask@2x.png", "PNG");
             //bitmap.setDevicePixelRatio(2);
             //mask.setDevicePixelRatio(2);
             EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
         }
         else {
-            bitmap = new QBitmap("scripts/CrosshairCursor.png", "PNG");
-            mask = new QBitmap("scripts/CrosshairCursorMask.png", "PNG");
+            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
+            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
             EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
         }
     }
@@ -836,6 +852,7 @@ EAction.getMainWindow = function() {
 /**
  * Reactivates the main window after showing a dialog.
  * This is necessary with Qt>=5.
+ * Not necessary with Qt>=5.10.
  */
 EAction.activateMainWindow = function() {
 //    if (RSettings.isQt(5)) {
